;-- TIME_STAMP   2020-06-13 01:41:50   v 0.1

#Region    ;************ Includes ************
#include-once
#include <WinAPIGdi.au3>
;~ #include <WinAPIDiag.au3> ; only for _WinAPI_DisplayStruct
#include <WinAPISysWin.au3>
#EndRegion ;************ Includes ************

; #FUNCTION# ====================================================================================================================
; Name ..........: _LoadWindowPlacement
; Description ...: Load (and set if $hWnd <> '') the saved placement of the window for Min, Max, and normal positions
; Syntax ........: _LoadWindowPlacement($sIniFile, $sWndName[, $hWnd = ''[, $iShowCmd = Default]])
; Parameters ....: $sIniFile - Name of the file in which the settings should be saved.
; ...............: $sWndName - Name of the settings to be loaded.
; ...............: $hWnd     - '' or Handle of the window for which the settings should be set.
; ...............: $iShowCmd - Overrides the current WindowState. (Only valid for the next WindowPlacement.)
; Return values .: Success: $tagWINDOWPLACEMENT structure with the placement coordinates
; ...............: Failure: False and set @error flag to non-zero
; ...............:          @error = 1 - no structure saved
; ...............:          @error = 2 - stored structure is faulty
; ...............:          @error = 3 - UBound($aMin) <> 2
; ...............:          @error = 4 - UBound($aMax) <> 2
; ...............:          @error = 5 - UBound($aNor) <> 4
; ...............:          @error = 6 - _WinAPI_SetWindowPlacement returns false
; Author ........: Bitnugger
; Remarks .......: If the information specified in WINDOWPLACEMENT would result in a window that is completely off the screen,
;                  the system will automatically adjust the coordinates so that the window is visible, taking into account
;                  changes in screen resolution and multiple monitor configuration.
; ===============================================================================================================================
Func _LoadWindowPlacement($sIniFile, $sWndName, $hWnd = '', $iShowCmd = Default)
	Local $sWindowPlacement = IniRead($sIniFile, $sWndName, 'WindowPlacement', '')
;~ 	If $sWindowPlacement = '' Then Return SetError(1, 0, False)
	If $sWindowPlacement = '' Then
;~ 		If IsHWnd($hWnd) Then WinSetState($hWnd, '', $iShowCmd = Default ? @SW_SHOW : $iShowCmd)
		If IsHWnd($hWnd) Then GUISetState($iShowCmd = Default ? @SW_SHOW : $iShowCmd, $hWnd)
		Return SetError(1, 0, False)
	EndIf
	Local $aWindowPlacement = StringSplit($sWindowPlacement, '|', 2)
	If UBound($aWindowPlacement) <> 6 Then Return SetError(2, 0, False)
	Local $aMin = StringSplit($aWindowPlacement[3], ',', 2), $aMax = StringSplit($aWindowPlacement[4], ',', 2), $aNor = StringSplit($aWindowPlacement[5], ',', 2)
	If UBound($aMin) <> 2 Then Return SetError(3, 0, False)
	If UBound($aMax) <> 2 Then Return SetError(4, 0, False)
	If UBound($aNor) <> 4 Then Return SetError(5, 0, False)

	Local $bRet = True, $tWindowPlacement = DllStructCreate($tagWindowPlacement)
	DllStructSetData($tWindowPlacement, 'length', $aWindowPlacement[0])
	DllStructSetData($tWindowPlacement, 'flags', $aWindowPlacement[1])
	DllStructSetData($tWindowPlacement, 'showCmd', $iShowCmd = Default ? $aWindowPlacement[2] : $iShowCmd)

	DllStructSetData($tWindowPlacement, 'ptMaxPosition', $aMin[0], 1)
	DllStructSetData($tWindowPlacement, 'ptMaxPosition', $aMin[1], 2)

	DllStructSetData($tWindowPlacement, 'ptMaxPosition', $aMax[0], 1)
	DllStructSetData($tWindowPlacement, 'ptMaxPosition', $aMax[1], 2)

	DllStructSetData($tWindowPlacement, 'rcNormalPosition', $aNor[0], 1)
	DllStructSetData($tWindowPlacement, 'rcNormalPosition', $aNor[1], 2)
	DllStructSetData($tWindowPlacement, 'rcNormalPosition', $aNor[2], 3)
	DllStructSetData($tWindowPlacement, 'rcNormalPosition', $aNor[3], 4)

	If IsHWnd($hWnd) Then
		$bRet = _WinAPI_SetWindowPlacement($hWnd, $tWindowPlacement)
;~ 		WinSetState($hWnd, '', $iShowCmd = Default ? $aWindowPlacement[2] : $iShowCmd)
		GUISetState($iShowCmd = Default ? $aWindowPlacement[2] : $iShowCmd, $hWnd)
	EndIf
	Return SetError($bRet ? 0 : 6, 0, $bRet ? $tWindowPlacement : False)
EndFunc   ;==>_LoadWindowPlacement

; #FUNCTION# ====================================================================================================================
; Name ..........: _SaveWindowPlacement
; Description ...: Save the placement of the window for Min, Max, and normal positions
; Syntax ........: _SaveWindowPlacement($sIniFile, $sWndName, $hWnd)
; Parameters ....: $sIniFile - Name of the file in which the settings should be saved.
; ...............: $sWndName - Name under which the settings should be saved.
; ...............: $hWnd     - '' or Handle of the window for which the settings should be set.
; Return values .: Success: True
; ...............: Failure: False and set @error flag to non-zero
; ...............:          @error = 1 - $hWnd is not a window handle
; ...............:          @error = 2 - _WinAPI_GetWindowPlacement returns false
; ...............:          @error = 3 - structure could not be saved
; Author ........: Bitnugger
; Remarks .......:
; ===============================================================================================================================
Func _SaveWindowPlacement($sIniFile, $sWndName, $hWnd)
	If Not IsHWnd($hWnd) Then Return SetError(1, 0, False)
	Local $sWindowPlacement = _GetWindowPlacement($hWnd, 'String')
	If @error Then Return SetError(2, 0, False)

	Local $iRet = IniWrite($sIniFile, $sWndName, 'WindowPlacement', $sWindowPlacement)
	Return SetError($iRet ? 0 : 3, 0, $iRet > 0)

;~ 	Local $tRectangle, $tWorkArea, $hMonitor = _WinAPI_MonitorFromRect($tNor), $aData = _WinAPI_GetMonitorInfo($hMonitor)
;~ 	If IsArray($aData) Then
;~ 		$tRectangle = DllStructCreate($tagRECT, DllStructGetPtr($aData[0]))
;~ 		$tWorkArea = DllStructCreate($tagRECT, DllStructGetPtr($aData[1]))
;~ 	Else
;~ 		$tRectangle = _WinAPI_GetWindowRect(_WinAPI_GetDesktopWindow())
;~ 		$tWorkArea = _WinAPI_GetWorkArea()
;~ 	EndIf
;~ 	Local $iRet _
;~ 		= IniWrite($sIniFile, $sWndName, 'Rectangle', _
;~ 			StringFormat('%s|%s|%s|%s', DllStructGetData($tRectangle, 'Left'), DllStructGetData($tRectangle, 'Top'), DllStructGetData($tRectangle, 'Right'), DllStructGetData($tRectangle, 'Bottom'))) _
;~ 		+ IniWrite($sIniFile, $sWndName, 'WorkArea', _
;~ 			StringFormat('%s|%s|%s|%s', DllStructGetData($tWorkArea, 'Left'), DllStructGetData($tWorkArea, 'Top'), DllStructGetData($tWorkArea, 'Right'), DllStructGetData($tWorkArea, 'Bottom')))
;~ 	Return SetError($iRet = 2 ? 0 : 3, 0, $iRet = 2)
EndFunc   ;==>_SaveWindowPlacement

; #FUNCTION# ====================================================================================================================
; Name ..........: _GetWindowPlacement
; Description ...: Get the placement of the window for Min, Max, and normal positions
; Syntax ........: _GetWindowPlacement($hWnd[, $vReturn = 'Both'])
; Parameters ....: $hWnd     - '' (active window) or Handle of the window for which the settings should be get.
; ...............: $vReturn  - [optional] a variant value. Default is 'Both'.
; ...............:
; Return values .: Success: True
; ...............:          $vReturn = 'String'    - $sWindowPlacement
; ...............:          $vReturn = 'DLLStruct' - $tWindowPlacement
; ...............:          $vReturn = 'Both'      - $vReturn[$tWindowPlacement, $sWindowPlacement]
; ...............: Failure: False and set @error flag to non-zero
; ...............:          @error = 1 - _WinAPI_GetWindowPlacement returns false
; Author ........: Bitnugger
; Remarks .......:
; ===============================================================================================================================
Func _GetWindowPlacement($hWnd, $vReturn = 'Both') ; 'String', 'DLLStruct', 'Both'
	If Not IsHWnd($hWnd) Then $hWnd = WinGetHandle('[ACTIVE]')
	Local $tWindowPlacement = _WinAPI_GetWindowPlacement($hWnd)
	If @error Then Return SetError(2, 0, False)
	Local $tMin = DllStructCreate($tagPOINT, DllStructGetPtr($tWindowPlacement, 'ptMinPosition'))
	Local $tMax = DllStructCreate($tagPOINT, DllStructGetPtr($tWindowPlacement, 'ptMaxPosition'))
	Local $tNor = DllStructCreate($tagRECT, DllStructGetPtr($tWindowPlacement, 'rcNormalPosition'))

	Local $sWindowPlacement = StringFormat('%s|%s|%s|%s,%s|%s,%s|%s,%s,%s,%s', _
		DllStructGetData($tWindowPlacement, 'length'), DllStructGetData($tWindowPlacement, 'flags'), DllStructGetData($tWindowPlacement, 'showCmd'), _
		DllStructGetData($tMin, 'X'), DllStructGetData($tMin, 'Y'), _
		DllStructGetData($tMax, 'X'), DllStructGetData($tMax, 'Y'), _
		DllStructGetData($tNor, 'Left'), DllStructGetData($tNor, 'Top'), DllStructGetData($tNor, 'Right'), DllStructGetData($tNor, 'Bottom'))
	Switch $vReturn
		Case 'String'
			Return $sWindowPlacement
		Case 'DLLStruct'
			Return $tWindowPlacement
		Case 'Both'
			Dim $vReturn[2]
			$vReturn[0] = $tWindowPlacement
			$vReturn[1] = $sWindowPlacement
			Return $vReturn
		Case Else
			Return SetError(3, 0, False)
	EndSwitch
EndFunc   ;==>_GetWindowPlacement

Func _CheckWinPos($hWnd, $bPrimary = False, $iOffset = 0)
	Local $iLeft, $iTop, $iWidth, $iHeight, $tPoint, $tWINDOWPLACEMENT, $hMonitor, $tMonitorRECT, $tWindowRECT

	$tWINDOWPLACEMENT = _WinAPI_GetWindowPlacement($hWnd)
	$tPoint = DllStructCreate($tagPOINT, DllStructGetPtr($tWINDOWPLACEMENT, 'rcNormalPosition'))
;~ 	_WinAPI_DisplayStruct($tPoint, $tagPoint, '$tPoint')
	$hMonitor = _WinAPI_MonitorFromPoint($tPoint, $MONITOR_DEFAULTTONULL)
	If $hMonitor <> 0x00000000 Then Return True
	ConsoleWrite('! _CheckWinPos: Fenster muss veschoben werden!' & @CRLF)
	$tWindowRECT = _WinAPI_GetWindowRect($hWnd)
	$hMonitor = _WinAPI_MonitorFromPoint($tPoint, $bPrimary ? $MONITOR_DEFAULTTOPRIMARY : $MONITOR_DEFAULTTONEAREST)
	$tMonitorRECT = _WinAPI_GetMonitorInfo($hMonitor)[0]
	$iLeft = DllStructGetData($tMonitorRECT, 'Left') + $iOffset
	$iTop = DllStructGetData($tMonitorRECT, 'Top') + $iOffset
	$iWidth = DllStructGetData($tWindowRECT, 'Right') - DllStructGetData($tWindowRECT, 'Left')
	$iHeight = DllStructGetData($tWindowRECT, 'Bottom') - DllStructGetData($tWindowRECT, 'Top')
	DllStructSetData($tWINDOWPLACEMENT, 'rcNormalPosition', $iLeft, 1)
	DllStructSetData($tWINDOWPLACEMENT, 'rcNormalPosition', $iTop, 2)
	DllStructSetData($tWINDOWPLACEMENT, 'rcNormalPosition', $iLeft + $iWidth, 3)
	DllStructSetData($tWINDOWPLACEMENT, 'rcNormalPosition', $iTop + $iHeight, 4)
;~ 	_WinAPI_DisplayStruct($tWINDOWPLACEMENT, $tagWINDOWPLACEMENT, '$tWINDOWPLACEMENT')
	_WinAPI_SetWindowPlacement($hWnd, $tWINDOWPLACEMENT)
EndFunc

